unit T_F4;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const
  Columns = 7;
  Rows    = 6;

type
  // Added type
  TBlockType    = (btRound, btSquare, btButton, btBeveled);
  TCpuIntelligence = (ciBaby,ciNormal,ciEinstein);
  TPlayerKind   = (plHuman, plComputer);
  TPlayer=RECORD
   PCOL:TCOLOR;
   KIND:TPlayerKind;
  END;
  TForza4Grid   = Array[0..Columns-1,0..Rows-1] of TColor;
  TWinGameEvent = procedure(Sender: TObject; Player: TPlayer) of Object;
  TF4 = class(TGraphicControl)
  private
   // Added properties
   fBlockType: TBlockType;
   fBitmap:    TBitmap;
   fLev:TCpuIntelligence;
   fPlaying: Boolean;
   PLAYER:ARRAY[1..2] OF TPLAYER;
   fWhosOnTurn:Integer;
   fGrid:TForza4grid;
   fLineColor,
   fBackColor:TColor;
   fDrawLines:Boolean;
   fOnWinGame:TWinGameEvent;
   fOnfalseMove:TNotifyEvent;
   HI1,HI2:TCOLOR;
   procedure Paint; override;
   procedure DrawBlock(X,Y: Integer;P:TCOLOR);
   procedure ClearGrid;
   Procedure SetBlockAt(X,Y:integer;P:TCOLOR);
  // procedure SetSampleBlocks;

   procedure fSetP1Color(C:TColor);
   procedure fSetP2Color(C:TColor);
   procedure fSetBackColor(C:TColor);
   procedure fSetLineColor(C:TColor);
   procedure fSetDrawLines(B:Boolean);
   procedure fSetBlockType(B:TBlockType);
   procedure fSetBitmap(Value: TBitmap);

   FUNCTION FindFirstFree(X:INTEGER):INTEGER;
   FUNCTION CheckForWinner(K:TCOLOR):BOOLEAN;
   Function RandomPosition:Integer;
   Function Find3(W:TCOLOR):Integer;
   Function Find2(W,OTHER,OUR:TCOLOR):Integer;  //W IS THE COLOR TO SEARCH
   Function Find1(W,OTHER,OUR:TCOLOR):Integer;  //W IS THE COLOR TO SEARCH
   Procedure CPUMove;
   FUNCTION NEXTPLAYER:INTEGER;
   FUNCTION PositionStatus(X,Y:INTEGER;K:TCOLOR):INTEGER;
   FUNCTION MAYWIN(M:INTEGER;C:TCOLOR):BOOLEAN;
  protected
    { Protected declarations }
  public
   procedure Click; override;
   procedure newGame;
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
    { Public declarations }
  published
   property WhoIsOnTurn:INTEGER read fWhosOnTurn write fWhosOnTurn;

   property Player1LOColor:TColor read PLAYER[1].PCOL write fSetP1Color default clBlue;
   property Player2LOColor:TColor read PLAYER[2].PCOL write fSetP2Color default clRed;
   property Player1HIColor:TColor read HI1 write HI1;
   property Player2HIColor:TColor read HI2 write HI2;
   property BackColor:TColor read fBackColor write fSetBackColor default clBlack;
   property LineColor: TColor read fLineColor write fSetLineColor default clSilver;
   property DrawLines: Boolean read fDrawLines write fSetDrawLines default True;
   Property CPULevel:TCpuIntelligence Read Flev Write FLev;
   // Added properties
   property BlockType: TBlockType read fBlockType write fSetBlockType default btRound;
   property BackBitmap: TBitmap read fBitmap write fSetBitmap;

   property Player1Kind: TPlayerkind read Player[1].KIND write Player[1].KIND;
   property Player2Kind: TPlayerkind read Player[2].KIND write Player[2].KIND;

   property OnWinGame: TWinGameEvent read fOnWinGame write fOnWinGame;
   property OnFalseMove: TNotifyevent read fOnFalsemove write fOnFalseMove;
  end;

procedure Register;

implementation

{$I CPUThink.PAS}

// Registratie

procedure Register;
begin
  RegisterComponents('Bianco', [TF4]);
end;

procedure TF4.DrawBlock(X,Y: Integer;P:TCOLOR);
Var Dx,Dy:real;
    C:TColor;
    X1,Y1,X2,Y2: Integer;

// Changed to respond to blocktype
Begin
    C:=P;

    Dx:=Width / Columns;
    Dy:=Height / Rows;

    X1:=Round(X*Dx)+1;Y1:=Round(Y*Dy)+1;
    X2:=Round((X+1)*Dx)-1;Y2:=Round((Y+1)*Dy)-1;

    With Canvas do
      begin
        Pen.Color:=C;
        Brush.Color:=C;
        Brush.Style:=bsSolid;
        If C=fBackColor then
          rectangle(X1,Y1,X2,Y2)
        else
        Case fBlockType of
        btRound:
          Ellipse(X1,Y1,X2,Y2);
        btSquare:
          rectangle(X1,Y1,X2,Y2);
        btButton:
          Begin
            rectangle(X1,Y1,X2,Y2);
            Pen.Color:=clBtnHighLight;
            MoveTo(X1,Y2);LineTo(X1,Y1);LineTo(X2,Y1);
            pen.Color:=clBtnShadow;
            LineTo(X2,Y2);LineTo(X1+1,Y2);
          End;
        btBeveled:
          begin
            rectangle(X1,Y1,X2,Y2);
            Pen.Color:=clBtnHighLight;
            MoveTo(X1,Y2);LineTo(X1,Y1);LineTo(X2,Y1);
            pen.Color:=clBtnShadow;
            LineTo(X2,Y2);LineTo(X1+1,Y2);

            Pen.Color:=clBtnShadow;
            MoveTo(X1+4,Y2-4);LineTo(X1+4,Y1+4);LineTo(X2-4,Y1+4);
            pen.Color:=clBtnHighLight;
            LineTo(X2-4,Y2-4);LineTo(X1+4,Y2-4);
          End;
        end;
     end;
End;

// Properties - write-methodes

procedure TF4.fSetBlockType(B:TBlockType);
begin
    fBlockType:=B;
    Paint;
end;

procedure TF4.fSetBitmap(Value: TBitmap);
Begin
    FBitmap.Assign(Value);
    Invalidate;
End;

procedure TF4.fSetP1Color(C:TColor);
begin
    if C<>PLAYER[2].PCOL then
     PLAYER[1].PCOL:=C
    else
      MessageDlg('The colors of the pieces cant be the same!',mtError,[mbRetry],0);
    Paint;
end;

procedure TF4.fSetP2Color(C:TColor);
begin
    if C<>PLAYER[1].PCOL then
     PLAYER[2].PCOL:=C
    else
      MessageDlg('The colors of the pieces cant be the same!',mtError,[mbRetry],0);
    Paint;
end;

procedure TF4.fSetBackColor(C:TColor);
begin
    fbackColor:=C;
    Paint;
end;

procedure TF4.fSetLineColor(C:TColor);
begin
    fLineColor:=C;
    Paint;
end;


procedure TF4.fSetDrawLines(B:Boolean);
begin
    fDrawlines:=B;
    Paint;
end;

//Constructor
constructor TF4.Create(AOwner:TComponent);
Begin
    Inherited create(AOwner);
    fBitmap:=TBitmap.Create;
    fPlaying:=False;
    PLAYER[1].PCOL:=clBlue;
    PLAYER[2].PCOL:=clRed;
    fBlockType:=btRound;
    fbackColor:=clBlack;
    fLineColor:=clSilver;
    FWHOSONTURN:=1;
    fDrawLines:=true;
    ClearGrid;
End;

destructor TF4.Destroy;
begin
    inherited destroy;
    fBitmap.Destroy;
end;


Procedure TF4.ClearGrid;
 Var
  X,Y:Integer;
 Begin
  For X:=0 to Columns-1 do
   For Y:=0 to Rows-1 do
    fGrid[X,Y]:=FBACKCOLOR;
 End;

// Paint - methode
procedure TF4.Paint;
Var Dx,Dy: real;
    X,Y:Integer;
Begin
    Dx:=Width / Columns;
    Dy:=Height / Rows;

    Canvas.Pen.Color:=fLineColor;
    Canvas.Brush.style:=bsSolid;
    Canvas.Brush.color:=fbackColor;
    Canvas.Rectangle(0,0,Width,Height);

    If fBitmap<>nil then
     Canvas.StretchDraw(ClientRect,fbitmap);
    //  Canvas.Draw(0,0,fBitmap);

    If fDrawLines then
     begin
      For X:=1 to Columns-1 do
        With Canvas do
          Begin
            MoveTo(Round(X*Dx),0); LineTo(Round(X*Dx),Height);
          End;
      For Y:=1 to Rows-1 do
        With Canvas do
          Begin
            MoveTo(0,Round(Y*Dy)); LineTo(Width,Round(Y*Dy));
          End;
     End;
    For X:=0 to Columns-1 do
      For Y:=0 to Rows-1 do
          If FGrid[X,Y]<>fBackColor then
            DrawBlock(X,Y,fGrid[X,Y]);
End;

procedure TF4.newGame;
Begin
 If (fPlaying=false) or (MessageDlg('Stop game in progess?',mtConfirmation,[mbYes,mbNo],0)=mrYes) then
  begin
   ClearGrid;
   RANDOMIZE;
   Paint;
   fPlaying:=true;
  end;
End;

Procedure TF4.SetBlockAt(X,Y:integer;P:TCOLOR);
 Begin
  fGrid[X,Y]:=P;
  DrawBlock(X,Y,P);
 End;

FUNCTION TF4.NEXTPLAYER:INTEGER;
 Begin
  //Change the player
  IF FWHOSONTURN=1 THEN
   RESULT:=2
  ELSE
   RESULT:=1;
 End;


END.
